#include "Gra.ch"
#include "Xbp.ch"
#include "Font.ch"
#include "Directry.ch"
#include "Common.ch"

#include "XCM.CH"

CLASS XCMBrowse FROM XbpBrowse
   EXPORTED:

      METHOD init
      METHOD create
      METHOD keyBoard
      METHOD DirArray()
      METHOD GuiArrayBrowse()
      METHOD ContextMenu()
      METHOD DirBlock()
      METHOD IconBlock()
      METHOD BMPBlock()
      METHOD SkipArray()
      METHOD myCell()
      METHOD STR2BLOCK()

      VAR Browser, aColElement, SearchTXT
      VAR cSeekTXT, currIndex, GroundTXT
      VAR escapePressed

      VAR aArray
      VAR LaufNo

ENDCLASS

METHOD XCMBrowse:init( oDraw,nNo)

   ::aArray := ::DirArray()

   ::Browser := ::GuiArrayBrowse( oDraw, , , PRES_PARAM, ::aArray )
   ::Browser:tabStop := .t.
*   ::Browser:keyBoard := { | nKey | ::KeyBoard( nKey ) }

   ::LaufNo := nNo

*   ::aColElement := aHeading[ 6 ]
*
*   ::GroundTXT := xbpStatic() :New( oParent,, { aPos[ 1 ], aPos[ 2 ] + 2 }, { aSize[ 1 ], 18 } )
*   ::GroundTXT:type := XBPSTATIC_TYPE_RAISEDRECT
*   ::GroundTXT:caption := "~S"
*
*   ::SearchTXT := xbpStatic() :New( oParent,, { aPos[ 1 ] + 1, aPos[ 2 ] + 3 }, { aSize[ 1 ] - 2, 16 }, aPresParam )
*   ::SearchTXT:caption := "Suchbegriff"
*   ::SearchTXT:options := XBPSTATIC_TEXT_CENTER + XBPSTATIC_TEXT_VCENTER
*   ::SearchTXT:setColorBG( GRA_CLR_BLUE )
*   ::SearchTXT:setColorFG( GRA_CLR_YELLOW )
*   ::cSeekTXT := ""

RETURN self

METHOD XCMBrowse:create()
LOCAL aHeading[ 6 ]
LOCAL i
LOCAL oMenu

   aHeading[ F_NAME          ] := { 14, "File name"     }
   aHeading[ F_SIZE          ] := {  4, "Size"          }
   aHeading[ F_WRITE_DATE    ] := {  4, "Date"          }
   aHeading[ F_WRITE_TIME    ] := {  4, "Time"          }
   aHeading[ F_ATTR          ] := {  1, "Attribute"     }

   ::browser:create()
*   ::GroundTXT:create()
*   ::SearchTXT:create()


   // First column displays file icons
*   ::Browser:addColumn ( ::IconBlock( ::aArray ), 1, "", , XBPCOL_TYPE_FILEMINIICON )
   ::Browser:addColumn ( ::IconBlock( ::aArray ), 1, "", , XBPCOL_TYPE_FILEICON )

   // Add XbpColumn objects for each column of the array
   FOR i:=1 TO 5
      ::Browser:addColumn( ::DirBlock( ::aArray, i ), aHeading[i,1], aHeading[i,2] )
   NEXT

   // Color for sorted column
   ::Browser:getColumn( F_NAME + 1 ):colorBlock := {|| { GRA_CLR_BLUE, NIL } }

   // Create context menu
   oMenu := ::ContextMenu( ::Browser )

   // Freeze first column
   ::Browser:setLeftFrozen( { 1 } )

   // Disallow browse cursor to enter first column after xbeK_LEFT
   ::Browser:navigate := ;
     {|mp1,mp2,obj| IIf( mp1 == XBPBRW_Navigate_PrevCol .AND. obj:colPos==2, NIL, mp1 ) }

   // Context menu is activated by a right mouse click
   ::Browser:itemRbDown := {|aPos,nRow,obj| oMenu:popUp( obj, aPos ) }

   // Cange directory after left double click or Enter key
   ::Browser:itemSelected := {|aPos,nRow,obj| ChangeDirectory( obj ) }

   // Display browser. It was hidden up to here. Therefore
   // the :show() method must be called explicitly in order
   // to get positions for columns and cells calculated.
   ::Browser:show()

   // Browse cursor starts in second column
   ::Browser:right()
   ::Browser:forceStable()

   i:=1
   FOR i:=1 TO 4
      ::Browser:getColumn(i+1):heading:lbClick := ::STR2BLOCK(i,::Browser,aHeading)
   NEXT

*   ::Browser:setInputFocus  := {| uNIL1, uNIL2, oSelf | oLW0 := ::LaufNo ,oSelf:refreshcurrent() }
   ::Browser:setInputFocus  := {| uNIL1, uNIL2, oSelf | SP_LW(::LaufNo) ,oSelf:Hilite() }
   ::Browser:killInputFocus := {| uNIL1, uNIL2, oSelf | oSelf:deHilite() }

   // Set focus to browser
   SetAppFocus( ::Browser )

RETURN self

METHOD XCMBrowse:keyBoard(nKey )

RETURN self

******************************************************************************
* Create an array which contains all data necessary to browse a 2-dim array
* - ARR_SRC : Data source (2-dim array = Directory())
* - ARR_NO  : 'record pointer'
* - ARR_ORD : Number of column for ASort()
* - ARR_SORT: 1-dim array containing an 'index' to ARR_SRC
******************************************************************************
METHOD XCMBrowse:DirArray()
LOCAL aArray[ ARR_COUNT ], nLen

   aArray[ ARR_SRC  ] := Directory( "*.*", "D" )
   aArray[ ARR_NO   ] := 1
   aArray[ ARR_ORD  ] := F_NAME
   aArray[ ARR_PATH ] := FullPath()

   // Create an array containing successive numbers (the 'index')
   nLen               := Len( aArray[ ARR_SRC ] )
   aArray[ ARR_SORT ] := AEval( Array(nLen), {|x,i| x:=i } ,,, .T. )

   // Sort successive numbers according to values of the data source array
   // (initial sort is by file name)
   ASort( aArray[ ARR_SORT ] ,,, ;
      {|i,j| aArray[ ARR_SRC, i, F_NAME ] < ;
             aArray[ ARR_SRC, j, F_NAME ] } )

RETURN aArray


******************************************************************************
* Create an XbpBrowse object with navigation code blocks for an array
* - aArray : This array contains the data source (2-dim array),
*            plus 'record pointer' and 'index' for the data source.
*            It is referenced in :cargo
******************************************************************************
METHOD XCMBrowse:GuiArrayBrowse( oParent, aPos, aSize, aPresParam, aArray )
   LOCAL oBrowse

   oBrowse := XbpBrowse():new( oParent, ,aPos, aSize, aPresParam )

   //:create()

   oBrowse:cargo         := aArray
   oBrowse:skipBlock     := {|n,obj| ::SkipArray( n, obj:cargo ) }
   oBrowse:goTopBlock    := {|obj| obj:cargo[ ARR_NO ] := 1 }
   oBrowse:goBottomBlock := {|obj| obj:cargo[ ARR_NO ] := Len( obj:cargo[ ARR_SRC ] ) }
   oBrowse:posBlock      := {|obj| obj:cargo[ ARR_NO ]  }
   oBrowse:phyPosBlock   := {|obj| obj:cargo[ ARR_SORT, obj:cargo[ ARR_NO ] ] }
   oBrowse:lastPosBlock  := {|obj| Len( obj:cargo[ ARR_SRC ] ) }
   oBrowse:firstPosBlock := {|obj| 1  }

   oBrowse:cursorMode    := XBPBRW_CURSOR_ROW   // Zeilen Cursor
   oBrowse:hScroll       := .F.

#IF XPPVER > 01900331
*   oBrowse:drawMode       := XBP_DRAW_OWNER      //ADVANCED
*   oBrowse:customDrawCell := {| oPS, aInfo, oSelf | ::myCell(oPS, aInfo, oSelf) }
#ENDIF

RETURN oBrowse

METHOD XCMBrowse:myCell(oPS, aInfo, oSelf)
#IF XPPVER > 01900331
LOCAL nRow    := aInfo[ XBP_DRAWINFO_ITEM   ] // Number of the row that contains the cell
LOCAL nAction := aInfo[ XBP_DRAWINFO_ACTION ] // Action or operation that caused xbeP_CustomDrawCell event
LOCAL nState  := aInfo[ XBP_DRAWINFO_STATE  ] // Current state of the cell
LOCAL aRect   := aInfo[ XBP_DRAWINFO_RECT   ] // Bounding rectangle for the drawing operation
LOCAL oArea   := aInfo[ XBP_DRAWINFO_AREA   ] // Column area (header, footer etc.) that contains the cell
LOCAL oCol    := aInfo[ XBP_DRAWINFO_COLUMN ] // XbpColumn object that contains the cell
LOCAL aAttr    := ARRAY( GRA_AS_COUNT )   // Verschiedene Attribute setzen

LOCAL cStr
LOCAL aItem
LOCAL nY
LOCAL aAAttrs   := ARRAY(GRA_AA_COUNT)
*LOCAL aLAttrs   := ARRAY(GRA_AL_COUNT)
LOCAL aSAttrs   := ARRAY(GRA_AS_COUNT)
LOCAL aOldAttrs
LOCAL nType
LOCAL aTxtBox
LOCAL aItemBG
LOCAL oBmp
LOCAL oIcon

LOCAL xCell, xCaption, aColor, xRet := TRUE

    IF nRow > oCol:RowCount() .OR. nRow < 1
       RETURN FALSE
    ENDIF

   aTxtBox := {aRect[1],aRect[2],aRect[3],aRect[4]}
   aItemBG := {aRect[1],aRect[2],aRect[3],aRect[4]}

/*

#define XBP_DRAWACTION_SELCHANGE       2
#define XBP_DRAWACTION_DRAWFG          32

#define XBP_DRAWACTION_DRAWALL         1
#define XBP_DRAWACTION_FOCUSCHANGE     4

#define XBP_DRAWACTION_DRAWBG          8
#define XBP_DRAWACTION_DRAWFRAME       16
#define XBP_DRAWACTION_ERASEBG         64
*/

   aItem := oCol:getRow(nRow)
   nType := oCol:type

*   IF "File name" $ aItem
*      TONE(100)
*   ENDIF

   IF BAND(nAction,XBP_DRAWACTION_DRAWALL) != 0  .AND. nRow == 1
*      TONE(100)
      nAction := BOr(nAction,XBP_DRAWACTION_SELCHANGE)
   ENDIF

   IF BAnd(nAction,XBP_DRAWACTION_SELCHANGE) != 0
*      TONE(100)
      IF BAnd(nState,XBP_DRAWSTATE_SELECTED) != 0
*      TONE(100)

      ENDIF
   ENDIF

*   aItem := oCol:getRow(nRow)
*   nType := oCol:type

   DO CASE
      CASE aItem = NIL
*ALTD()

      CASE nType = XBPCOL_TYPE_TEXT         // *)  Die Spalte zeigt Text und Ziffern an

*         oCol:styleClass   := "TAB"
*         oCol:stylePart    := 1
**         oCol:controlState := XBP_STATE_HOT
*         oCol:controlState := XBP_STATE_PRESSED
*
*//       GraBackground( <oPS>, [<aStart>], [<aEnd>], [<lErase>], [<nPart>] )
*         GraBackground( oPS , {aTxtBox[1],aTxtBox[2]},{aTxtBox[3],aTxtBox[4]},.T.,3)
*
**         aSAttrs[ GRA_AS_HORIZALIGN ] := GRA_HALIGN_RIGHT
**         aSAttrs[ GRA_AS_VERTALIGN ]  := GRA_VALIGN_HALF
**         aSAttrs[ GRA_AS_ANGLE ]      := {1,0}
**        aSAttrs[ GRA_AS_COLOR ]      := IIF(BAnd(nState,XBP_DRAWSTATE_DISABLED) != 0,XBPSYSCLR_MENUDISABLEDTEXT,XBPSYSCLR_WINDOWTEXT)
*

         aSAttrs[ GRA_AS_COLOR ]      := XBPSYSCLR_WINDOWTEXT
         aOldAttrs                    := GraSetAttrString(oPS,aSAttrs)

         IF VALTYPE(aItem) = "C"
            GraCaptionStr(oPS,{aTxtBox[1],aTxtBox[2]},;
                              {aTxtBox[3],aTxtBox[4]},aItem)

         ELSEIF VALTYPE(aItem) = "N"
            GraCaptionStr(oPS,{aTxtBox[1],aTxtBox[2]},;
                              {aTxtBox[3],aTxtBox[4]},Transform( aItem,"@N"))

         ELSEIF VALTYPE(aItem) = "D"
            GraCaptionStr(oPS,{aTxtBox[1],aTxtBox[2]},;
                              {aTxtBox[3],aTxtBox[4]},DTOC( aItem) )

         ELSEIF VALTYPE(aItem) = "L"
            GraCaptionStr(oPS,{aTxtBox[1],aTxtBox[2]},;
                              {aTxtBox[3],aTxtBox[4]},IIF(aItem, ".T.", ".F."))

         ELSEIF VALTYPE(aItem) = "M"
            GraCaptionStr(oPS,{aTxtBox[1],aTxtBox[2]},;
                              {aTxtBox[3],aTxtBox[4]},"Memo")
         ENDIF


      CASE nType = XBPCOL_TYPE_BITMAP       //  Die Spalte zeigt Bitmaps an


      CASE nType = XBPCOL_TYPE_ICON         //  Die Spalte zeigt Icons an

      CASE nType = XBPCOL_TYPE_SYSICON      //  Die Spalte zeigt System-Icons an

      CASE nType = XBPCOL_TYPE_FILEICON     //  Die Spalte zeigt Datei-Icons an

*         aItem:draw(oPS,{aRect[1],aRect[2]})
          oCol:DataArea:SetCell( nRow, xCaption, oCol:type,.T.)

      CASE nType = XBPCOL_TYPE_FILEMINIICON //  Die Spalte zeigt kleine Datei-Icons an

          xCaption := xCell := oCol:DataArea:GetCell( nRow )

          IF xCell == NIL .OR. xCell != xCaption ;
                          .OR. oCol:Type == XBPCOL_TYPE_FILEICON ;
                          .OR. oCol:Type == XBPCOL_TYPE_FILEMINIICON

/*
            oCol:styleClass   := "TAB"
            oCol:stylePart    := 4
            oCol:controlState := XBP_STATE_HOT
            oCol:controlState := XBP_STATE_AUTO

            GraBackground( oPS , {aTxtBox[1],aTxtBox[2]},{aTxtBox[3],aTxtBox[4]},.T.,3)
*/
*   ALTD()
            IF VALTYPE(xCaption) = "N"
               TONE(100)
            ENDIF

            xRet := oCol:DataArea:SetCell( nRow, xCaption, oCol:type,.F.)


*            xRet := oCol:DataArea:setCell(nRow,"..", XBPCOL_TYPE_FILEMINIICON)
*            oCG:setCell( "c:\windows\..", XBPCOL_TYPE_FILEICON )
*             oCol:DataArea:InvalidateRect()
          ENDIF


   ENDCASE

*  aLAttrs                 := ARRAY(GRA_AL_COUNT)
*  aLAttrs[ GRA_AL_COLOR ] := COLOR_BASE_XFUER
*  GraSetAttrLine(oPS,aLAttrs)
*  nY := aTxtBox[2]+(aTxtBox[4] - aTxtBox[2]) / 2
*
*  GraLine(oPS,{aItemBG[1]+ITEM_SPACING,nY},{aItemBG[3] - ITEM_SPACING * 2,nY})

#ENDIF

RETURN self

******************************************************************************
* Create a context menu to select sorting order
******************************************************************************
METHOD XCMBrowse:ContextMenu( oBrowse )
   LOCAL oMenu := XbpMenu( oBrowse ):new():create()

   oMenu:addItem( { " ~Name ", {|| SortFiles( oBrowse, F_NAME       ) } } )
   oMenu:addItem( { " ~Size ", {|| SortFiles( oBrowse, F_SIZE       ) } } )
   oMenu:addItem( { " ~Date ", {|| SortFiles( oBrowse, F_WRITE_DATE ) } } )
   oMenu:addItem( { " ~Time ", {|| SortFiles( oBrowse, F_WRITE_TIME ) } } )

RETURN oMenu


******************************************************************************
* Return a :dataLink code block for the array created by DirArray()
* - nSubScript is a detached LOCAL variable.
*   It determines the column of the data source to display
******************************************************************************
METHOD XCMBrowse:DirBlock( aArray, nSubScript )
RETURN {|nElement| nElement := aArray[ ARR_NO ], ;
                   nElement := aArray[ ARR_SORT, nElement ], ;
                   aArray[ ARR_SRC, nElement, nSubScript ] }


******************************************************************************
* Return a :dataLink code block for the array created by DirArray()
* It returns a full qualified file name -> to display file icons
******************************************************************************
METHOD XCMBrowse:IconBlock( aArray )
RETURN {|nElement| nElement := aArray[ ARR_NO ], ;
                   nElement := aArray[ ARR_SORT, nElement ], ;
                   aArray[ ARR_SRC, nElement, F_NAME ] }

METHOD XCMBrowse:BMPBlock( aArray )


RETURN {|| NIL }

******************************************************************************
* Function to maintain the 'record pointer' for arrays
* It is called via oBrowse:skipBlock
******************************************************************************
METHOD XCMBrowse:SkipArray( nWantSkip, aArray )
   LOCAL nDidSkip, nLastRec := Len( aArray[ ARR_SRC ] )

   IF aArray[ ARR_NO ] + nWantSkip < 1              // "BoF"
      nDidSkip := 1 - aArray[ ARR_NO ]
   ELSEIF aArray[ ARR_NO ] + nWantSkip > nLastRec   // "EoF"
      nDidSkip := nLastRec - aArray[ ARR_NO ]
   ELSE
      nDidSkip := nWantSkip
   ENDIF

   aArray[ ARR_NO ] += nDidSkip

RETURN nDidSkip

METHOD XCMBrowse:STR2BLOCK(nIndexNo,oBrowse,aColElement)
LOCAL i
LOCAL nCol := oBrowse:colCount

   FOR i := 1 TO nCol
      IF i = 1
      ELSE
         oBrowse:getColumn(i):heading:setCell( 1, aColElement[i-1,2] )
         oBrowse:getColumn(i):heading:inValidateRect()
         oBrowse:getColumn(i):heading:setColorFG( GRA_CLR_WHITE )
         oBrowse:getColumn(i):heading:setColorBG( GRA_CLR_BLUE )
      ENDIF
   NEXT
   oBrowse:getColumn( nIndexNo ):heading:setCell( 1, aColElement[ nIndexNo, 2 ] + " /\" )
   *   oBrowse:deHilite()
   oBrowse:colpos := nIndexNo
   oBrowse:getColumn( nIndexNo ):heading:setColorFG( GRA_CLR_YELLOW )
   oBrowse:getColumn( nIndexNo ):heading:setColorBG( GRA_CLR_RED )

   DO CASE
      CASE nIndexNo = 1 ; SortFiles( oBrowse, F_NAME )
      CASE nIndexNo = 2 ; SortFiles( oBrowse, F_SIZE )
      CASE nIndexNo = 3 ; SortFiles( oBrowse, F_WRITE_DATE )
      CASE nIndexNo = 4 ; SortFiles( oBrowse, F_WRITE_TIME )
   ENDCASE

   *   oBrowse:Hilite()
   oBrowse:refreshAll()

RETURN
